// PROJECT  :SQWTimerScheduling
// PURPOSE  :Offers a 'General' outline for how to schedule tasks using interrupts
//          :SQW pulses are counted and displayed at required intervals
// AUTHOR   :C. D'Arcy
// DATE     :2023 01 30
// uC       :328P/DS1307
// STATUS   :Working
// REFERENCE:http://darcy.rsgc.on.ca/ACES/Datasheets/DS1307.pdf
//          :Support for 2022/23 ICS3U-E I2C Data Logger Project
#include <Wire.h>
#define RTCADDRESS  0x68      //
#define SQWREGISTER 7         //
#define SQWE        4         //
#define RS0         0         //
#define RS1         1         //

#define Hz1         0<<RS1|0<<RS0   // 1 Hz
#define Hz4k        0<<RS1|1<<RS0   // 4.096 kHz (2^12)
#define Hz8k        1<<RS1|0<<RS0   // 8.192 kHz (2^13)
#define Hz32k       1<<RS1|1<<RS0   // 32.768 kHz (2^15)

#define INTERRUPTPIN 2
volatile uint32_t count = 0;      //ensure large enough integer variable

#include <TimerOne.h>
#define INTERVAL    2000000       //
volatile boolean triggered = false;


void setup() {
  /*  pinMode(A2, OUTPUT);        //required for use with DS1307 as Appliance
      pinMode(A3, OUTPUT);
      digitalWrite(A2, LOW);
      digitalWrite(A3, HIGH);*/
  Serial.begin(9600);
  while (!Serial);
  Serial.println("SQW Interrupts over " + String(INTERVAL) + " ms.");

  attachInterrupt(digitalPinToInterrupt(INTERRUPTPIN), ISR_SQW, FALLING);
  pinMode(INTERRUPTPIN, INPUT_PULLUP);    //SQW pin is an open-collector
  Wire.begin();                           //
  setSQW(Hz1);                          // Hz1, Hz4k, Hz8k, Hz32k
  Timer1.initialize(INTERVAL);            //
  Timer1.attachInterrupt(ISR_Timer1);     //
}

// Periodic reporting interval reached
void ISR_Timer1 () {
  triggered = true;
}

// Accumulate the number of falling edges of the DS1307 square wave
void ISR_SQW () {
  count++;
}

//Wire Level communication to set the SQW frequency
uint8_t setSQW(uint8_t frequency) {
  Wire.beginTransmission(RTCADDRESS);
  Wire.write(SQWREGISTER);
  Wire.write(1 << SQWE | frequency); // RS1:RS0=00 selects 1Hz
  Wire.endTransmission();
  // optional: read it back for verification/confirmation...
  Wire.beginTransmission(RTCADDRESS);
  Wire.write(SQWREGISTER);
  Wire.endTransmission();
  Wire.requestFrom(RTCADDRESS, 1);
  while (!Wire.available());
  uint8_t value = Wire.read();
  return value;
}

void loop() {
  if (triggered) {            //report the count at interval
    triggered = false;
    Serial.println(count);
    count = 0;                //reset the count
  }
}
